@@ -37,13 +37,20 @@
  * \author Cedric Bastoul and Louis-Noel Pouchet
  */
 
-# include <stdlib.h>
-# include <stdio.h>
-# include <string.h>
-# include <candl/candl.h>
+#include <stdlib.h>
+#include <stdio.h>
+#include <string.h>
+#include <candl/candl.h>
 
 #include <assert.h>
 
+#ifdef CANDL_SUPPORTS_ISL
+# undef Q // Thank you polylib...
+# include <isl/int.h>
+# include <isl/constraint.h>
+# include <isl/ctx.h>
+# include <isl/set.h>
+#endif
 
 /******************************************************************************
  *                          Structure display function                        *
@@ -318,6 +325,60 @@ candl_program_deps_to_string(CandlDependence* dependence)
   return buffer;
 }
 
+#ifdef CANDL_SUPPORTS_ISL
+
+struct isl_set*
+isl_set_from_piplib_matrix(struct isl_ctx* ctx,
+			   PipMatrix* matrix,
+			   int nparam);
+PipMatrix*
+isl_set_to_piplib_matrix(struct isl_ctx* ctx,
+			 struct isl_set* set,
+			 int nparam);
+/**
+ * candl_dependence_isl_simplify function:
+ *
+ * This function uses ISL to simplify the dependence polyhedra.
+ * Useful for polyhedra that contain large coefficient values.
+ *
+ */
+CandlDependence* candl_dependence_isl_simplify(CandlDependence* dep,
+					       CandlProgram* program)
+{
+  if (dep == NULL || program == NULL)
+    return dep;
+
+  CandlDependence* tmp;
+  PipMatrix* context = (PipMatrix*) program->context;
+  int nparam = context->NbColumns - 2;
+
+  struct isl_ctx* ctx = isl_ctx_alloc ();
+
+  for (tmp = dep; tmp; tmp = tmp->next)
+    {
+      // 1- Convert the dependence polyhedron into ISL set.
+      struct isl_set* set =
+	isl_set_from_piplib_matrix(ctx, tmp->domain, nparam);
+
+      // 2- Simplify the ISL set.
+      set = isl_set_detect_equalities(set);
+
+      // 3- Convert back into Candl matrix representation.
+      PipMatrix* newdom = isl_set_to_piplib_matrix(ctx, set, nparam);
+      isl_set_free (set);
+      candl_matrix_free (tmp->domain);
+      tmp->domain = (CandlMatrix*) newdom;
+    }
+
+  /// FIXME: Some dead ref.
+  //isl_ctx_free (ctx);
+
+  return dep;
+}
+
+#endif
+
+
 
 /**
  * candl_dependence_print_scop function:
@@ -504,6 +565,7 @@ CandlDependence* candl_dependence_read_from_scop(scoplib_scop_p scop,
   char* deps = scoplib_scop_tag_content(scop,
 					"<dependence-polyhedra>",
 					"</dependence-polyhedra>");
+
   /* No dependence, nothing to do. */
   if (deps == NULL)
     return NULL;
@@ -519,11 +581,12 @@ CandlDependence* candl_dependence_read_from_scop(scoplib_scop_p scop,
   for (; *deps != '\n'; ++deps);
   ++deps;
   for (i = 0; *deps != '\n'; ++i, ++deps)
-    buffer_nb[i] = deps[i];
+    buffer_nb[i] = *deps;
   buffer_nb[i] = '\0';
   ++deps;
   int nbdeps = atoi (buffer_nb);
   char* next;
+
   /* For each of them, read 1 and shift of the read size. */
   for (depcount = 0; depcount < nbdeps; ++depcount)
     {
@@ -1270,7 +1333,6 @@ candl_dependence_p candl_dependence(candl_program_p program,
 
   statement = program->statement;
   context = program->context;
-
   if (options->scalar_privatization || options->scalar_expansion)
     candl_dependence_analyze_scalars (program, options);
 
@@ -1526,7 +1588,8 @@ candl_dependence_compute_lb (CandlMatrix* m, Entier* lb, int col)
   if ((solution != NULL) &&
       ((solution->list != NULL) || (solution->condition != NULL)))
     {
-      for (l = solution->list; l && col > 0; l = l->next, --col)
+      l = solution->list;
+      for (; l && col > 0; l = l->next, --col)
 	;
       CANDL_assign(*lb, l->vector->the_vector[0]);
     }
@@ -1923,7 +1986,6 @@ candl_dependence_is_loop_carried (candl_program_p program,
 				  int loop_index)
 {
   int i, j;
-
   /* Ensure source and sink share common loop 'loop_index', and that
      dependence depth is consistent with the considered loop. */
   for (i = 0; i < dep->source->depth; ++i)
@@ -1945,9 +2007,10 @@ candl_dependence_is_loop_carried (candl_program_p program,
   CANDL_set_si(m->p[m->NbRows - 2][i + 1], -1);
   CANDL_set_si(m->p[m->NbRows - 1][dep->source->depth + 1 + j], -1);
   /* Copy the rest of the matrix. */
-  for (i = 0; i < dep->domain->NbRows; ++i)
-    for (j = 0; j < dep->domain->NbColumns; ++j)
-      CANDL_assign(m->p[i][j], dep->domain->p[i][j]);
+  int ii, jj;
+  for (ii = 0; ii < dep->domain->NbRows; ++ii)
+    for (jj = 0; jj < dep->domain->NbColumns; ++jj)
+      CANDL_assign(m->p[ii][jj], dep->domain->p[ii][jj]);
   /* Compute real lb of loops. */
   Entier lb; CANDL_init(lb);
   candl_dependence_compute_lb (m, &lb, i + 1);
